import torch
from .interface import BaseExplainerInterface

class TabularExplainer(BaseExplainerInterface):
    """
    

    :param BaseExplainerInterface: [description]
    :type BaseExplainerInterface: [type]
    """    

    def __init__(self):
        super(self.__class__,self).__init__()

        